#import "RNSelectableTextRectHelper.h"
#import <React/RCTUIManager.h>
#import <React/RCTUIManagerUtils.h>
#import <React/UIView+React.h>

#import "RNSelectableTextView.h"

#if __has_include(<RCTText/RCTUITextView.h>)
#import <RCTText/RCTUITextView.h>
#else
#import "RCTUITextView.h"
#endif

@implementation RNSelectableTextRectHelper

RCT_EXPORT_MODULE();

RCT_EXPORT_METHOD(getRectForSubstring:(nonnull NSNumber *)reactTag startOffset:(nonnull NSNumber *)start endOffset:(nonnull NSNumber *)end resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    RCTUIManager *uiManager = self.bridge.uiManager;

    dispatch_async(uiManager.methodQueue, ^{
        [uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *,UIView *> *viewRegistry) {
            
            RNSelectableTextView *_backedTextInputView = (RNSelectableTextView *)viewRegistry[reactTag];
            if (![[_backedTextInputView class] isSubclassOfClass:[RNSelectableTextView class]]) {
                reject(@"E_INVALID_VIEW", @"The view is not a RNSelectableTextView", nil);
                return;
            }
            
            RCTUITextView *textView = [_backedTextInputView backedTextInputView];
          
            UITextPosition *startPosition = [textView positionFromPosition:textView.beginningOfDocument offset:start.intValue];
            UITextPosition *endPosition = [textView positionFromPosition:textView.beginningOfDocument offset:end.intValue];
            
            if (!startPosition || !endPosition) {
                reject(@"E_INVALID_POSITION", @"Invalid start or end position", nil);
                return;
            }
            
            UITextRange *textRange = [textView textRangeFromPosition:startPosition toPosition:endPosition];

            NSArray<UITextSelectionRect *> *selectionRects = [textView selectionRectsForRange:textRange];
            NSMutableArray *rects = [NSMutableArray arrayWithCapacity:selectionRects.count];
            for (UITextSelectionRect *selectionRect in selectionRects) {
                CGRect rect = selectionRect.rect;
                [rects addObject:@{
                    @"x": @(rect.origin.x),
                    @"y": @(rect.origin.y),
                    @"width": @(rect.size.width),
                    @"height": @(rect.size.height),
                }];
            }
            
            resolve(@{
                @"rects": rects
            });

        }];
    });
}


@end
